#pragma once
#include <avr/pgmspace.h>
#include "avr/sleep.h"
#include <avr/power.h>
#include <EEPROM.h>
#include "settings.h"
#include "pins.h"
#include "TimerOne.h"

void ditPaddleIRQ();
void dahPaddleIRQ();
void ditPaddlePressedIRQ();
void dahPaddlePressedIRQ();

class KeyerIO
{
  public:
  
  enum MODE
  {
    IAMBIC_A,
    IAMBIC_B,
    CMOS,
    ULTIMATIC
  };

  private:
  
  enum MODE          m_Mode;
  bool               m_DotDashMemory;
  
  unsigned char      m_CurrentWPM;
  unsigned int       m_SideToneFrequency;
  
  unsigned long int  m_DitMillis;
  unsigned long int  m_DahMillis;
  unsigned long int  m_Ratio;
  
  bool          m_LastSymbolWasDit;
  bool          m_SideToneOn;
  bool          m_ReversePaddle;
  bool          m_UseSpeedRatio;
  bool          m_DoNotTransmit;
  
  public:

  void init( bool reset );
  
  void setRatio( unsigned int ratio, bool correctForWPM=false );
  
  void setWPM( unsigned int speedWPM );
  unsigned char getWPM();
  
  void setSideToneFrequency( unsigned int frequency );
  unsigned int getSideToneFrequency();

  bool getTXState();
  void setTXState( bool state );
  
  unsigned int getDitMillis();
  unsigned int getDahMillis();

  void setMode( enum MODE mode );

  void sendSymbol( bool isDit = true );
  void sendDit();
  void sendDah();
  void sendAlternating();
  void sendChar( unsigned char c );
  void sendString( unsigned char const * const string );
  void sendStringNoTX( unsigned char const * const string );
  
  void sendNumber( int number );
  void sendNumberNoTX( int number );
  
  void bleepUp();
  void bleepDown();
  void bleepOK();
  void bleepError();

  void toggleDotDashMemory();
  void toggleDotDashMemory( bool state );
  
  void toggleSideTone();
  void toggleSideTone( bool state );
  
  void toggleReversePaddle();
  void toggleReversePaddle( bool state );

  void toggleSpeedRatio();
  void toggleSpeedRatio( bool state );
  
  bool isSideToneOn();
  bool isReversePaddle();
  bool isSpeedRatioOn();
  bool isDotDashMemoryOn();
  
  int checkButtons();
  void checkWPMPoti( bool init=false );
  
  void eepromStoreWpm();
  void eepromStoreSideTone();
  void eepromStoreFrequency();
  void eepromStoreReversePaddle();
  void eepromStoreSpeedRatio();
};

